# awips_tkgm --
#
#	This file provides the geomap_awips package, which accesses data
# 
# Copyright (c) 2003, 2004 Gordon D. Carrie.  All rights reserved.
# 
# Licensed under the Open Software License version 2.1
# 
# Please address questions and bug reports to user0@tkgeomap.org.
# 
#  @(#) $Id: awips.tcl,v 1.17 2004/09/22 22:37:14 tkgeomap Exp $

if {[catch "package present Tk"] == 0} {
    package require tkgeomap 2
} else {
    package require tclgeomap 2
}
package provide geomap_awips 2.1

# Make sure require executables are available.

if [catch {exec which dbfdump} errMsg] {
    error "$errMsg.  dbfdump application is required to access shape files"
}
if [catch {exec which shptobin} errMsg] {
    error "$errMsg.  shptobin is required to access shape files"
}

namespace eval geomap::awips::county {}
namespace eval geomap::awips::county::center {}
namespace eval geomap::awips::state {}
namespace eval geomap::awips::state::center {}

# geomap::awips::county::read --
#
#	This procedure loads selected county outlines from the "AWIPS Counties"
#	shapefile.
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.
  
proc geomap::awips::county::read {shapefile {region -all} args} {
    variable counties
    variable state
    variable name
    variable fips

    set nmspc [namespace current]

    # Determine the region

    set useState 0
    set useCounty 0
    set useAll 0
    set useLatLon 0
    switch -- $region {
	-all {
	    set useAll 1
	    puts "Reading all counties"
	}
	-states {
	    set useState 1
	    eval set states $args
	    puts "Reading counties from states: $states"
	}
	-bounds {
	    eval set bounds $args
	    if {[scan $bounds {%f %f %f %f} north east south west] != 4
		    || $south >= $north || $west >= $east} {
		error "Bad bounds: $bounds"
	    }
	    set useLatLon 1
	    puts "Reading counties in bounds $bounds"
	}
	default {
	    error "Bad region specifier: $region"
	}
    }

    # Read the .dbf file

    set in [open "|dbfdump $shapefile" r]
    set dbf_list {}
    gets $in
    while {[gets $in line] > 0} {
	lappend dbf_list $line
    }
    close $in

    # Load the desired counties

    set i 0
    set outlines {}
    set centers {}
    set counties {}
    foreach dbf_line [lrange $dbf_list 0 end] {
	
	# Get information about the county from the data base entry

	set st [string range $dbf_line 0 1]
	set cnty [string trimright [string range $dbf_line 16 37]]
	set ips [string range $dbf_line 41 45]
	if {[string length $st] == 0 || [string length $cnty] == 0
		|| [string length $ips] == 0} {
	    puts stderr \
		    "WARNING: Could not get identifiers from:\n$dbf_line\n"
	    continue
	}
	if {[scan [string range $dbf_line 72 101] { %f %f} lon lat] != 2} {
	    puts stderr "WARNING: Could not get lat lon from:\n$dbf_line\n"
	    continue
	}
	set id c$i

	# If county is in region of interest, create a linearray and place
	# for it.

	if {   $useAll
            || ($useState && [lsearch $states $st] >= 0)
	    || ($useLatLon && $south <= $lat && $lat <= $north
		&& [::geomap::lonbtwn $lon [list $west $east]]) } {
	    set outline ${nmspc}::$id
	    ::geomap::lnarr fmbin $outline "|shptobin -s -i $i $shapefile"
	    $outline descr [list $cnty $st]
	    set state($id) $st
	    set name($id) $cnty
	    set fips($id) $ips
	    lappend counties $id
	    set center ${nmspc}::center::$id
	    ::geomap::place new $center [list $lat $lon]
	}
	incr i
    }
    return [llength $counties]
}

# geomap::awips::county::regexp --
#
#	This procedure searches county names for a pattern.
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::awips::county::regexp {exp {nocase 0}} {
    variable counties
    variable name

    set nc [expr {$nocase ? "-nocase" : "--"}]
    set rslt {}
    foreach c $counties {
	if [::regexp $nc $exp $name($c)] {
	    lappend rslt $c
	}
    }
    return $rslt
}

# geomap::awips::state::read 
#
#	This procedure loads state outlines from the NWS/AWIPS
#	"U.S. States and Territories" shapefile.
#
# Arguments:
#	See the user documentation.
#	shapefile	an ESRI shapefile, composed of the following files:
#		  	${shapefile}.dbf, ${shapefile}.shp, ${shapefile}.shx. 
#
# Results:
#	See the user documentation.

proc geomap::awips::state::read {shapefile} {
    variable states
    variable name

    set nmspc [namespace current]
    set in [open "|dbfdump $shapefile" r]
    set dbf_list {}
    gets $in
    while {[gets $in line] > 0} {
	lappend dbf_list $line
    }
    close $in
    set i 0
    foreach dbf_line [lrange $dbf_list 1 end] {
	set st [string range $dbf_line 0 1]]
	set nm [string trim [string range $dbf_line 6 30]]
	if {[scan [string range $dbf_line 33 end] { %f %f} lon lat] != 2} {
	    error "Could not get lat lon from:\n$dbf_line"
	}
	::geomap::lnarr fmbin ${nmspc}::$st "|shptobin -s -i $i $shapefile"
	$st descr "$nm outline from NWS/AWIPS"
	set name($st) $nm
	lappend states $st
	::geomap::place new ${nmspc}::center::$st [list $lat $lon]
	incr i
    }
    return [llength $states]
}
