# us_census.tcl
#
#	This file defines the geomap_census_gaz package, that accesses and
#	manipulates data from the U.S. Census gazetteer.
#	Ref. http://www.census.gov/geo/www/tiger/index.html
# 
# Copyright (c) 2003, 2004 Gordon D. Carrie.  All rights reserved.
# 
# Licensed under the Open Software License version 2.1
# 
# Please address questions and bug reports to user0@tkgeomap.org.

if {[catch "package present Tk"] == 0} {
    package require tkgeomap 2
} else {
    package require tclgeomap 2
}
package provide geomap_census_gaz 2.1

# This block of code initializes namespace variables.  See the user
# documentation for meanings.

namespace eval geomap::us_census::gaz {
    variable name
    variable state
    variable type
    variable pop
    variable places
    variable n_places
    variable types
}

# geomap::us_census::gaz::read --
#
#	This procedure reads U.S. Census gazetteer data.
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::us_census::gaz::read {inFile {verbose 0}} {
    variable name
    variable state
    variable type
    variable pop
    variable places
    variable n_places
    variable types

    set nmspc [namespace current]
    if $verbose {
	puts "Reading place data"
    }
    set in [open $inFile r]
    while {[gets $in line] >= 0} {

	# Read information for a place

	set st [string range $line 0 1]
	set fips [string range $line 2 8]
	set place [string range $line 9 72]
	if {[scan [string range $line 73 81] {%d} population ] != 1} {
	    error "Could not get population from $line"
	}
	if {[scan [string range $line 143 163] {%f %f} lat lon] != 2} {
	    error "Could not get lat lon from $line"
	}

	# Store information for the place

	::geomap::place new ${nmspc}::$fips [list $lat $lon]
	if {[llength $place] > 1} {
	    set type($fips) [string tolower [lindex $place end]]
	    set name($fips) [string trim [lreplace $place end end]]
	} else {
	    set type($fips) "unknown"
	    set name($fips) [string trim $place]
	}
	set state($fips) $st
	lappend types $type($fips)
	set pop($fips) $population
	lappend places $fips
    }
    close $in

    if $verbose {
	puts "Sorting place data"
    }
    set places [lsort -command popcmp -decreasing $places]
    foreach place $places {
	lappend n_places ${nmspc}::$place
    }
    set types [lsort -unique $types]
    return [llength $places]
}

# geomap::us_census::gaz::read_sorted --
#
#	This procedure reads place data from an optimized file
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::us_census::gaz::read_sorted {inFile} {
    variable name
    variable state
    variable type
    variable pop
    variable places
    variable n_places
    variable types

    set nmspc [namespace current]
    set in [open $inFile r]
    while {[gets $in line] >= 0} {
	set id [lindex $line 0]
	set name($id) [lindex $line 1]
	set state($id) [lindex $line 2]
	set latLon [lindex $line 3]
	::geomap::place new ${nmspc}::$id $latLon
	set type($id) [lindex $line 4]
	lappend types $type($id)
	set pop($id) [lindex $line 5]
	lappend places $id
	lappend n_places ${nmspc}::$id
    }
    close $in
    set types [lsort -unique $types]
    return [llength $places]
}

# geomap::us_census::gaz::popcmp --
#
#	This procedure compares two places by population.
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::us_census::gaz::popcmp {p1 p2} {
    variable pop
    expr {$pop($p1) > $pop($p2) ? 1 : $pop($p1) < $pop($p2) ? -1 : 0}
}

# geomap::us_census::gaz::regexp --
#
#	This procedure searches the list of place names for a pattern
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::us_census::gaz::regexp {exp {nocase 0}} {
    variable places
    variable name

    set nc [expr {$nocase ? "-nocase" : "--"}]
    set rslt {}
    foreach plc $places {
	if [::regexp $nc $exp $name($plc)] {
	    lappend rslt $plc
	}
    }
    return $rslt
}

# geomap::us_census::gaz::smallest --
#
#	This procedure finds the smallest place with a population greater
#	than a given value.
#
# Arguments:
#	minPop	- population value
#
# Results:
#	Returns the index of the place in places (or n_places) with the
#	smallest population that is still greater than pop.

proc geomap::us_census::gaz::smallest {minPop} {
    variable places
    variable pop

    set i 0
    foreach place $places {
	if {$pop($place) >= $minPop} {
	    incr i
	} else {
	    break
	}
    }
    return [expr {$i - 1}]
}
