# usgs_gaz.tcl --
#
#	This file provides the geomap_usgs_gaz package, which accesses place
#	data from the U.S. Geological Survey gazetteer at
#	http://mapping.usgs.gov/www/gnis
# 
# Copyright (c) 2003, 2004 Gordon D. Carrie.  All rights reserved.
# 
# Licensed under the Open Software License version 2.1
# 
# Please address questions and bug reports to user0@tkgeomap.org.
# 
#  @(#) $Id: usgs_gaz.tcl,v 1.13 2004/09/22 22:37:15 tkgeomap Exp $

if {[catch "package present Tk"] == 0} {
    package require tkgeomap 2
} else {
    package require tclgeomap 2
}
package provide geomap_usgs_gaz 2.1

# Create the namespace

namespace eval geomap::usgs::gaz {}

# geomap::usgs::gaz::read --
#
#	This proceudre reads place data from a USGS file.
#	See http://mapping.usgs.gov/pub/gnis/00README.html for information
#	on the file formats.
#
# Arguments:
#	See the user documentation.
#
# Results:
#	See the user documentation.

proc geomap::usgs::gaz::read {fileNm type} {

    # Initialize the type namespace

    InitType $type

    # Read the file

    set fl [open $fileNm r]
    while {[gets $fl line] >= 0} {
	${type}::readcol $line
    }
    close $fl
    set l [namespace eval $type {llength $places}]
    if {$l == 0} {
	namespace delete $type
    }
    return $l
}

# geomap::usgs::gaz::InitType --
#
#	This procedure initializes a type namespace
#
# Arguments:
#	type	- place type from USGS gazetteer
#
# Results:
#	Namespace procedures and variables are initialized.

proc geomap::usgs::gaz::InitType {type} {
    namespace eval $type {
	set type [namespace tail [namespace current]]
	proc readcol line $geomap::usgs::gaz::ReadCol
	proc regexp {exp {nocase 0}} $geomap::usgs::gaz::RegexpScript
	if ![info exists np] {
	    set np 0
	}
	if ![info exists places] {
	    set places {}
	}
    }
}

# geomap::usgs::gaz::ReadCol --
#
#	This script is the body for a procedure in a type namespace.
#	The procedure should be created with a call equivalent to:
#
# 	        proc geomap::usgs::gaz::typeName::readcol {line} \
#			$geomap::usgs::gaz::ReadCol
#
#	The resulting procedure parses a line from a USGS gazetteer
#	"Columnar Record Format" file.
#
# Arguments:
#	line 	line from the file
#
# Results:
#	If line contains data for a place whose type matches that of the
#	namespace, the place data is stored in the namespace.

set geomap::usgs::gaz::ReadCol {
    variable n_places
    variable places
    variable name
    variable county
    variable state
    variable elev
    variable np

    # Parse the line

    set nmspc [namespace current]
    set type [string trim [string range $line 54 62]]
    if {$type != [namespace tail $nmspc]} {
	return
    }
    set st [string trim [string range $line 0 1]]
    set nm [string trim [string range $line 3 52]]
    set cnty [string trim [string range $line 64 78]]
    if {[scan [string trim [string range $line 80 95]] \
	    {%2d%2d%2d%s %3d%2d%2d%s} latd latm lats ns lond lonm lons ew] \
	    != 8} {
	error "Could not read lat-lon from $line"
    }
    set el [string range $line 114 119]

    # Store line data

    set placeId p$np
    set latLon [::geomap::dmstodec \
	    [list $latd $latm $lats $ns $lond $lonm $lons $ew]]
    if {[catch [list ::geomap::place new ${nmspc}::$placeId $latLon] msg]
	    != 0} {
	error $msg
    }
    lappend n_places ${nmspc}::$placeId
    lappend places $placeId
    set name($placeId) $nm
    set county($placeId) $cnty
    set state($placeId) $st
    set elev($placeId) $el
    incr np
}

# geomap::usgs::gaz::RegexpScript --
#
#	This script is the body for a procedure in a type namespace.
#	The procedure should be created with a call equivalent to:
#
#	proc geomap::usgs::gaz::typeName::regexp exp {nocase 0} \
#			$geomap::usgs::gaz::RegexpScript
#
#	The resulting procedure will search for names in the type
#	namespace that match a pattern.
#
# Arguments:
#	See the user documentation for the regexp command.
#
# Results:
#	See the user documentation for the regexp command.

set geomap::usgs::gaz::RegexpScript {
    variable places
    variable name

    set nc [expr {$nocase ? "-nocase" : "--"}]
    set rslt {}
    foreach p $places {
	if [::regexp $nc $exp $name($p)] {
	    lappend rslt $p
	}
    }
    return $rslt
}
